#  Perl 5 required.
require 5.000;

#  version EN 1.10 h
#  a-changes Feb 1 1998
#    edit and delete checks if page notarized
#    annotate sketchpad works like edit sketchpad
#    annotate and image upload leave mime type unchanged
#    notarize date placed in history and marked at end of page
#    notarize reloads updated page.
#  b-changes Feb 10 1998
#    sketchpad points to correct notebook in multi-notebook setups
#  c-changes Feb 25 1998
#    fixed typo causing annotate to be active in read-only mode
#    edit now correctly uploads plain and html files
#  d-changes April 21 1998
#    added ability to turn off control panel
#  e-changes May 8 1998
#    added ability to add captions to uploaded images in ADD or EDIT
#  f-changes June 11 1998  (thanks to Martin Beaudoin)
#    added code to fix large-file upload bug in some NT-Perl versions
#    added a confirm dialog to notarize and delete buttons
#  g-changes May 14 1999
#    added last-modified date to TOC entries
#  h-changes April 14 2000
#    added ability to turn off notarize
#    

#  Strict syntax checking.
use strict;

# Add this for wrapping plain text on 80 characters (gave errors so commented out)
#use Text::Wrap qw(wrap $columns);
#$columns = 80;


=head1 NAME

ENoteLib - electronic notebook shared library routines

=head1 SYNOPSIS

    require 'enotelib.pl';

=head1 DESCRIPTION

=head1 AUTHOR

Noel Nachtigal E<lt>F<santa@msr.epm.ornl.gov>E<gt>
Al Geist E<lt>F<geist@msr.epm.ornl.gov>E<gt>

=cut

###############################################################################
#
#  Miscellaneous global flags and variables.
#
###############################################################################

#  File locking constants.
$main::LOCK_SH = 1;			# Shared lock
$main::LOCK_EX = 2;			# Exclusive lock
$main::LOCK_NB = 4;			# Non-blocking lock
$main::LOCK_UN = 8;			# Unlock

#  Return codes.
$main::RET_NO_ERROR = 0;		# No error
$main::RET_BAD_SRVR = 1;		# Bad server environment
$main::RET_BAD_RQST = 2;		# Bad HTTP request
$main::RET_BAD_NTBK = 3;		# Invalid notebook requested
$main::RET_BAD_PAGE = 4;		# Invalid page or object number
$main::RET_BAD_FILE = 5;		# File error

###############################################################################
#
#  This is main.
#
###############################################################################

sub main {

#  Build the URL for the notebook script.
  if ( "$ENV{'SERVER_NAME'}" eq '' ) {
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Invalid Environment</title>
</head>
<body background="$main::backgrnd">
<h1>Invalid Server Environment</h1>
<p><b>The server accessed does not provide an environment that is fully
compliant with the CGI protocol.<br>
Please contact the Web server administrator for further details.</b><br>
Please use your browser to go back.<hr> 
</body>
</html>
EOF
    exit $main::RET_BAD_SRVR;
}


  my $https = ( "$ENV{'HTTPS'}" ne '' ) ? 's' : '';
  my $port = "$ENV{'SERVER_PORT'}";
  if ( "$port" eq "80" ) {
    $https = ''; $port = '';
  }
  elsif ( "$port" eq "443" ) {
    $https = 's'; $port = '';
  }
  my $name = "$ENV{'SERVER_NAME'}";

###########################=mtest=###########################
## set variables for use in sketchpad links in add & edit  ##
##                                                         ##
     $main::server    = "$name";                           ##
     $main::engine    = "$main::script";                   ##
                                                           ##
     @main::codebase  = split('/', $main::gifs);           ##
     pop(@main::codebase);                                 ##
     $main::codebase  = join('/', @main::codebase);        ##
     $main::codebase  = "$main::codebase/sketchpad/";      ##
                                                           ##
     @main::sketchpad = split ('/', $main::script);        ##
     pop(@main::sketchpad);                                ##
     $main::sketchpad = join('/', @main::sketchpad);       ##
     $main::sketchpad = "$main::sketchpad/sketchpad.pl";  ##
##                                                         ##
###########################=mtest=###########################


  if ( "$name" ne '' ) {
    $name = "//$name" . (( "$port" ne '' ) ? ":$port" : '' );
  }

  $main::script = "http$https:$name$main::script";

#  Get the request method; only accept GET and POST requests.
  $main::method = uc( $ENV{'REQUEST_METHOD'} );
  if ( $main::method !~ m/^(GET|POST)$/ ) {
    $main::method = '(null)' if ( "$main::method" eq '' );
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Communication Error</title>
</head>
<body background="$main::backgrnd">
<h1>Communication Error</h1>
<p><b>Your browser sent me a request I do not understand.
I was expecting a GET or a POST, and I got a "$main::method".</b><br>
Please use your browser to go back.<hr>
</body>
</html>
EOF
    exit $main::RET_BAD_RQST;
  }

#  Parse the parameters.
  &ParseCGIParms();

#  Extract the notebook name and info.
  $main::nb = $main::in{'nb'} if ( "$main::in{'nb'}" ne '' );
  $main::nb = lc( $main::nb );
  if ( ! $main::notebook{$main::nb} ) {
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Invalid Notebook</title>
</head>
<body background="$main::backgrnd">
<h1>Invalid Notebook</h1>
<p><b>The notebook requested ($main::nb) cannot be found or is not accessible.</b><br>
Please use your browser to go back.<hr>
</body>
</html>
EOF
    exit $main::RET_BAD_NTBK;
  }
  $main::root	 = $main::notebook{$main::nb}{'dir'};
  $main::title	 = $main::notebook{$main::nb}{'title'};
  
#  Make sure the index file exists and is OK.
  $main::idxfile = "$main::root/index.lst";
  if ( ! -e $main::idxfile ) {
    open( IDX, ">$main::idxfile" ) || &InvalidFile();
    close( IDX );
  }

#  Extract the global page number; default is 'last'.
  $main::page = ( "$main::in{'page'}" eq '' ) ? 'last' :
		lc( $main::in{'page'} );

#  Extract the action --- it's case-insensitive, default is 'view'.
  my $action = ( "$main::in{'action'}" eq '' ) ? 'view' :
	       lc( $main::in{'action'} );
  $action = 'view' if
    ( $action !~ m/^(add|annotate|delete|edit|notarize|search|toc|view)$/ );

#  If 'cancel', then we really want just 'view'.
  $action = 'view' if ( $main::in{'cancel'} );

  if	( "$action" eq 'add' )	    { &ActionAdd(); }
  elsif ( "$action" eq 'annotate' ) { &ActionAnnotate(); }
  elsif ( "$action" eq 'delete' )   { &ActionDelete(); }
  elsif ( "$action" eq 'edit' )     { &ActionEdit(); }
  elsif ( "$action" eq 'notarize' ) { &ActionNotarize(); }
  elsif ( "$action" eq 'search' )   { &ActionSearch(); }
  elsif ( "$action" eq 'toc' )	    { &ActionToc(); }
  elsif ( "$action" eq 'view' )     { &ActionView(); }

  exit $main::RET_NO_ERROR;
}

###############################################################################
#
#  Primary notebook function routines
#
###############################################################################

###############################################################################
#
#  ( ) = &ActionAdd();
#
#  Adds an entry to the notebook.
#
###############################################################################

sub ActionAdd {

#  Extract info if available.
  my ( $author, $body, $kwords, $mimetype, $title ) = ( '', '', '', '' );
  my ( $caption ) = ('');
  my $isentry = 1;

  if ( "$main::in{'reset'}" ne '' ) {
    $main::method = 'GET';
  }
  else {
    $author   = "$main::in{'author'}"; $author =~ s/"/&quot;/g;
    $kwords   = lc( "$main::in{'keywords'}" ); $kwords =~ s/"/&quot;/g;
    $mimetype = "$main::mimetypes{$main::in{'mimetype'}}";
    $mimetype = "$main::up{'upfile'}{'mime'}"
      if ( "$main::up{'upfile'}{'mime'}" ne '' );
    $isentry  = ( "$mimetype" =~ m;^text/(html|plain)$; );
    $title    = "$main::in{'title'}"; $title =~ s/"/&quot;/g;
    $caption  = "$main::in{'caption'}";
    if ( $isentry ) {
      $body = "$main::in{'body'}"; $body =~ s/\r/ /g; $body =~ s/\0//;
    }
    $body = $main::up{'upfile'}{'data'} if ( $main::up{'upfile'}{'size'} > 0 );
  }

#  GET:  we have either a new form, or a cancelled form.
#  In either case, put up a fresh form.
  if ( "$main::method" eq 'GET' ) {
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Add a Notebook Entry</title>
</head>
<body background="$main::backgrnd">
$main::linehorz

<h2>Add a Notebook Entry to a New Page</h2>
<p>
This form allows the user to choose from three different 
input methods. <br>
<b>Text/HTML,</b> or <b> Quick Sketch</b>, or <b>File/Image  upload</b>.
</p>
Fill out <b>Name</b> and <b>Title</b> (required).

EOF
  }

#  POST: we have a submitted form, either a notebook entry or an object.
#  It might be incomplete.
  elsif ( $author && $body && $mimetype && $title ) {	# Complete, save it

    if ( ! $isentry ) {				# Notebook object
      my $date = time;
      my $strdate = gmtime( $date );
      $date = substr( $date, 0, length( $date ) - 3 ) . '.' .
	      substr( $date, -3 );
      my $filename = "$main::root/$date";
      open( FILE, ">$filename" ) || &InvalidFile();
      binmode( FILE );
      print FILE $body;
      close( FILE );
      chmod 0666, ( $filename );
      my $objs = 0;
      open( IDX, "<$main::idxfile" ) || &InvalidFile();
      flock( IDX, $main::LOCK_EX ) if ( $main::flock );
      seek( IDX, 0, 0 );
      while ( <IDX> ) {
	next if ( m/^\s*$/ );			# Blank lines
	next if ( m/^\s*#/ );			# Comment lines
	my ( $tag ) = split;
	$objs++ if ( $tag < 0 );
      }
      $objs++;
      flock( IDX, $main::LOCK_UN ) if ( $main::flock );
      close( IDX );
      open( IDX, ">>$main::idxfile" ) || &InvalidFile();
      flock( IDX, $main::LOCK_EX ) if ( $main::flock );
      print IDX "-$objs $date $mimetype 0\n";
      flock( IDX, $main::LOCK_UN ) if ( $main::flock );
      close( IDX );
      sleep( 1 );
      if ( "$mimetype" =~ m;^image/(jpeg|gif)$; ) {
	$body = <<EOF;
<p>
<center>
<img src="$main::script?nb=$main::nb&action=view&page=-$objs">
<br><b> $caption </b>
</center>
</p>
EOF
      }
      else {
	$body = <<EOF;
<p>
Click <a href="$main::script?nb=$main::nb&action=view&page=-$objs">here</a>
to view the object ($caption) you uploaded.
</p>
EOF
      }
      $mimetype = 'text/html';
    }

# Notebook entry
#  Make new filename from the time.
    my $date = time;
    my $strdate = gmtime( $date );
    $date = substr( $date, 0, length( $date ) - 3 ) . '.' .
	    substr( $date, -3 );
    my $filename = "$main::root/$date";
    open( FILE, ">$filename" ) || &InvalidFile();
    print FILE <<EOF;
<!-- Author: $author -->
<!-- Date: $strdate (GMT) -->
<!-- Title: $title -->
<!-- Keywords: $kwords -->
<!-- Changes: Added $strdate (GMT) $author | -->
$body
EOF
    close( FILE );
    chmod 0666, ( $filename );

#  Open the index file and count number of entries.
    my $pages = 0;
    open( IDX, "<$main::idxfile" ) || &InvalidFile();
    flock( IDX, $main::LOCK_EX ) if ( $main::flock );
    seek( IDX, 0, 0 );
    while ( <IDX> ) {
      next if ( m/^\s*$/ );			# Blank lines
      next if ( m/^\s*#/ );			# Comment lines
      my ( $tag ) = split;
      $pages++ if ( $tag > 0 );
    }
    $pages++;
    flock( IDX, $main::LOCK_UN ) if ( $main::flock );
    close( IDX );

#  Add new file to the index file.
    open( IDX, ">>$main::idxfile" ) || &InvalidFile();
    flock( IDX, $main::LOCK_EX ) if ( $main::flock );
    print IDX "$pages $date $mimetype 0\n";
    flock( IDX, $main::LOCK_UN ) if ( $main::flock );
    close( IDX );

#  Redirect the browser to the new entry.
    print "Location: $main::script?nb=$main::nb&action=view&page=last\n\n";
    exit $main::RET_NO_ERROR;
  }

  else {					# Incomplete entry, retry
    $body = '' if ( ! $isentry );
    if ( "$mimetype" ne '' ) {
      foreach ( sort keys %main::mimetypes ) {
	$main::def_type = "$_" if ( "$main::mimetypes{$_}" eq "$mimetype" );
      }
    }
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Incomplete Entry</title>
</head>
<body background="$main::backgrnd">
$main::linehorz

<h2>Incomplete Entry</h2>

Every notebook entry must have an author, a title, and some text.<br>
Please complete the missing parts.

EOF
  }

#  The rest of the 'Add' form.


  print <<EOF;
<form action="$main::script" enctype="multipart/form-data" method=post>
<p>
<pre>
<b>Your name</b>: <input type=text name=author value="$author"
 size=40>
<b>The title</b>: <input type=text name=title value="$title"
 size=40>
<b>Keywords</b> : <input type=text name=keywords value="$kwords"
 size=40> (optional)
</pre>
</p>
<p>
You can <b>type</b> or <b>cut and paste</b> text or HTML 
into the text area (it will scroll to accommodate the input):<br>
<textarea rows=15 cols=75 name=body>$body</textarea><br>
</p>

Or you can <a href="$main::sketchpad?codebase=$main::codebase&server=$main::server&engine=$main::engine&amp;notebook=$main::nb&page=last">

<b>Draw and add a sketch</b></a> to a new page in the notebook.

<p>
Or you can <b>upload a file</b> (eg. image) to a new page:
<input type=file name=upfile value="" size=20>
<br><input type=text name=caption value="$caption" size=40>
image caption (optional)
</p>
<p>
<b>Choose the MIME type</b> of your input:
<SELECT NAME="mimetype">
EOF

  foreach ( sort keys %main::mimetypes ) {
    print '<OPTION';
    print ' SELECTED' if ( "$_" eq "$main::def_type" );
    print "> $_</OPTION>\n";
  }

  print <<EOF;
</SELECT>
</p>
<p>
Then submit your entry:<br>
<input type=submit name=submit value="Add this entry">
<input type=submit name=reset value="Start over">
<input type=submit name=cancel value="Cancel">
<input type=hidden name=action value="add">
<input type=hidden name=nb value="$main::nb">
<input type=hidden name=page value="$main::page">
</p>
</form>
</p>
</body>
</html>
EOF

  return;
}

###############################################################################
#
#  ( ) = &ActionAnnotate();
#
#  Annotates an entry in the notebook.
#  NEW - shows copy of the page and a text box inwhich to place annotation.
#
###############################################################################

sub ActionAnnotate {

#  Initialize variables.
  my ( $authorig, $body, $changes, $kwords, $mimetype, $title ) =
     ( '', '', '', '', '', '' );
  my ( $author, $text, $date ) = ( '', '', '' );
  my ( $base, $filename, $update ) = ( '', '', 0 );

#  Open index file and extract file name and mimetype
  open( IDX, "<$main::idxfile" ) || &InvalidFile();
  flock( IDX, $main::LOCK_EX ) if ( $main::flock );
  seek( IDX, 0, 0 );
  my @lines = <IDX>;
  foreach ( @lines ) {
    next if ( m/^\s*$/ );			# Blank lines
    next if ( m/^\s*#/ );			# Comment lines
    my ( $tag, $name, $type ) = split;
    if ( "$tag" eq "$main::page" ) {
      $base = "$name"; $mimetype = "$type"; last;
    }
  }
  flock( IDX, $main::LOCK_UN ) if ( $main::flock );
  close( IDX );

#  Set update if mimetype isn't text/html
  $filename = lc( $mimetype );
  $mimetype = "$main::mimetypes{$main::in{'mimetype'}}" if
    ( "$main::in{'mimetype'}" ne '' );
  $mimetype = lc( $mimetype );
  $update = ( "$filename" ne "$mimetype" );
  $filename = "$main::root/$base";

#  Request for input page. Open file extracting existing page content.
  if ( "$main::method" eq 'GET' ) {
    open( FILE, "<$filename" ) || &InvalidFile();
    while ( <FILE> ) {
      if    ( m/^<!-- Author: (.*) -->$/ )   { $authorig = $1; }
      elsif ( m/^<!-- Date: (.*) -->$/ )     { $date = $1; }
      elsif ( m/^<!-- Title: (.*) -->$/ )    { $title = $1; }
      elsif ( m/^<!-- Keywords: (.*) -->$/ ) { $kwords = $1; }
      elsif ( m/^<!-- Changes: (.*) -->$/ )  { $changes = $1; }
      else { $body .= $_; }
    }
    close( FILE );

#  Build header of normal input page
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Annotate $main::page</title>
</head>
<body background="$main::backgrnd"> 
<center>
<h2>Annotate page $main::page </h2>
<h3>Insert note at bottom and submit</h3>
</center>
$main::linehorz
EOF
  }
#  else assume POST: we have a submitted annotation form.
  else {
    $author   = "$main::in{'author'}"; $author =~ s/"/&quot;/g;
    $text     = "$main::in{'text'}"; $text =~ s/\r/ /g; $text =~ s/\0//;

    if ( $author && $text ) {	# Information complete. 

#  Append to existing file.
      my $strdate = gmtime( time );
      open( FILE, ">>$filename" ) || &InvalidFile();
      print FILE <<EOF;
<p><hr>
<font color=RED><b><u>Addendum</u>:</b></font>&nbsp;&nbsp; <b>$author</b> &nbsp; Date: <b>$strdate</b> (GMT) <br>
$text
<hr>
EOF
      close( FILE );
      chmod 0666, ( $filename );

#  If the MIME type changed, update the index file.
      if ( $update ) {
        open( IDX, "<$main::idxfile" ) || &InvalidFile();
        flock( IDX, $main::LOCK_EX ) if ( $main::flock );
        seek( IDX, 0, 0 );
        my @lines = <IDX>;
        flock( IDX, $main::LOCK_UN ) if ( $main::flock );
        close( IDX );
        open( IDX, ">$main::idxfile" ) || &InvalidFile();
        flock( IDX, $main::LOCK_EX ) if ( $main::flock );
        seek( IDX, 0, 0 );
        foreach ( @lines ) {
          next if ( m/^\s*$/ );			# Blank lines
          next if ( m/^\s*#/ );			# Comment lines
          my ( $tag, $name, $type, $as_is ) = split;
          print IDX ( "$tag" eq "$main::page" ) ?
	    "$tag $base $type $as_is\n" : "$_";
        }
        flock( IDX, $main::LOCK_UN ) if ( $main::flock );
        close( IDX );
      }

#  Redirect the browser to the new entry.
      print <<EOF;
Location: $main::script?nb=$main::nb&action=view&page=$main::page

EOF
      exit $main::RET_NO_ERROR;
    }

# Incomplete entry, build retry header
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Incomplete Annotation</title>
</head>
<body background="$main::backgrnd">
$main::linehorz

<center><h2>Incomplete Annotation</h2></center>

An annotation must have an author and some text.<br>
Please press <b>Back</b> on your browser and<br>
complete the missing parts.
<br>
</body>
</html>
EOF
      exit $main::RET_NO_ERROR;
  }
#  Set the default MIME type.
  if ( "$mimetype" ne '' ) {
    foreach ( sort keys %main::mimetypes ) {
      $main::def_type = "$_" if ( "$main::mimetypes{$_}" eq "$mimetype" );
    }
  }

# Finish building the input page for both normal and incomplete entry cases.


#  Build the existing page body.
  $changes =~ s/\|/<br>/g;
  if ( "$changes" ne '' ) { print "\n<h4>$changes</h4>\n"; }
  else { print "\n<h4>$date</h4>\n"  if ( "$date" ne '' ); }
  print "\n<h4>Keywords: $kwords</h4>\n"     if ( "$kwords" ne '' );
  print "\n<center><h3>$title</h3></center>" if ( "$title" ne '' );
  print "\n<p>";
  print "\n<pre>"  if ( "$mimetype" eq 'text/plain' );
  #
  # Wrap plain text within 80 characters if text/plain data
  #if ( "$mimetype" eq 'text/plain' ){
  #  print "\n"; print wrap("", "", $body);
  #}
  #else {
    print "\n$body";
  #}
  print "\n</pre>" if ( "$mimetype" eq 'text/plain' );
  print "\n</p><p>";
 
# ADD FORM HERE
  print <<EOF;
<form action="$main::script" enctype="multipart/form-data" method=post>
<p>
<pre>
<b>Your name</b>: <input type=text name=author value="$author"
 size=40>
</pre>
</p>
<p>
You can <b>type</b> or <b>cut and paste</b> text or HTML 
into the text area (it will scroll to accommodate the input):<br>
<textarea rows=5 cols=75 name=text>$text</textarea><br>
</p>

Or you can <a href="$main::sketchpad?codebase=$main::codebase&server=$main::server&engine=$main::engine&amp;notebook=$main::nb&page=$main::page">
<b>Draw and add a sketch</b></a> to the end of this page.

<p>
Then submit your entry:<br>
<input type=submit name=submit value="Add this annotation">
<input type=submit name=cancel value="Cancel">
<input type=hidden name=action value="annotate">
<input type=hidden name=mimetype value="text/html">
<input type=hidden name=nb value="$main::nb">
<input type=hidden name=page value="$main::page">
</p>
</form>
</p>
</body>
</html>
EOF

  return;
}

###############################################################################
#
#  ( ) = &ActionDelete();
#
#  Deletes a notebook page.
#
###############################################################################

sub ActionDelete {

#  Open the index file and find our entry.
  open( IDX, "<$main::idxfile" ) || &InvalidFile();
  flock( IDX, $main::LOCK_EX ) if ( $main::flock );
  seek( IDX, 0, 0 );
  my @lines = <IDX>;
  flock( IDX, $main::LOCK_UN ) if ( $main::flock );
  close( IDX );
  open( IDX, ">$main::idxfile" ) || &InvalidFile();
  flock( IDX, $main::LOCK_EX ) if ( $main::flock );
  seek( IDX, 0, 0 );
  foreach ( @lines ) {
    next if ( m/^\s*$/ );			# Blank lines
    next if ( m/^\s*#/ );			# Comment lines
    my ( $tag, $name, $type, $as_is ) = split;
    print IDX ( ("$tag" eq "$main::page") && ($as_is == 0) ) ? "$tag\n" : "$_";
  }
  flock( IDX, $main::LOCK_UN ) if ( $main::flock );
  close( IDX );

#  Display deleted page.
  print <<EOF;
Location: $main::script?nb=$main::nb&action=view&page=$main::page

EOF

  return ;
}

###############################################################################
#
#  ( ) = &ActionEdit();
#
#  Edits an entry to the notebook.
#
###############################################################################

sub ActionEdit {

#  Extract info if available.

  if ( "$main::in{'reset'}" ne '' ) {
    print <<EOF;
Location: $main::script?nb=$main::nb&action=view&page=$main::page

EOF
    return;
  }

  my ( $author, $body, $changes, $kwords, $mimetype, $title ) =
     ( '', '', '', '', '', '' );
  my ( $base, $filename, $notarized, $update ) = ( '', '', 0, 0 );
  my ( $pagetype ) = ( '' );
  my ( $caption  ) = ( '' );


  open( IDX, "<$main::idxfile" ) || &InvalidFile();
  flock( IDX, $main::LOCK_EX ) if ( $main::flock );
  seek( IDX, 0, 0 );
  my @lines = <IDX>;
  foreach ( @lines ) {
    next if ( m/^\s*$/ );			# Blank lines
    next if ( m/^\s*#/ );			# Comment lines
    my ( $tag, $name, $type, $as_is ) = split;
    if ( "$tag" eq "$main::page" ) {
      $base = "$name"; $mimetype = "$type"; $notarized = $as_is; last;
    }
  }
  flock( IDX, $main::LOCK_UN ) if ( $main::flock );
  close( IDX );

  $filename = lc( $mimetype );
  $mimetype = "$main::mimetypes{$main::in{'mimetype'}}" if
    ( "$main::in{'mimetype'}" ne '' );
  $mimetype = "$main::up{'upfile'}{'mime'}" if
    ( "$main::up{'upfile'}{'mime'}" ne '' );
  $mimetype = lc( $mimetype );
  $update = ( "$filename" ne "$mimetype" );
  $filename = "$main::root/$base";

  if ( ("$main::method" eq 'GET') && ($notarized == 0)) {
    open( FILE, "<$filename" ) || &InvalidFile();
    while ( <FILE> ) {
      if    ( m/^<!-- Author: (.*) -->$/ )   { $author = $1; }
      elsif ( m/^<!-- Date: (.*) -->$/ )     { ; }
      elsif ( m/^<!-- Title: (.*) -->$/ )    { $title = $1; }
      elsif ( m/^<!-- Keywords: (.*) -->$/ ) { $kwords = $1; }
      elsif ( m/^<!-- Changes: (.*) -->$/ )  { $changes = $1; }
      else { $body .= $_; }
    }
    close( FILE );
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Edit a Notebook Entry</title>
</head>
<body background="$main::backgrnd">
$main::linehorz

<h2>Edit the Notebook Entry on This Page</h2>
<p>
This form allows the you to choose from three different
methods to modify this page. <br>
You can change or add too the existing <b>Text/HTML,</b><br>
or you can append a <b> Quick Sketch</b> to the end of the page,<br>
or you can <b>upload a File/Image</b> to the end of the page.
<p>
The <b>Name</b> and <b>Title</b> are required.

EOF
  } elsif ( ("$main::method" eq 'GET') && ($notarized == 1)) {
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Edit invalid</title>
</head>
<body background="$main::backgrnd">
$main::linehorz

<h2>Notarized Page Can Not be Edited</h2>
<p>
The requested page has been previously notarized.<br>
Notarized pages can not be edited, but they can be annotated.
</body>
</html>
EOF
  }


#  POST: we have a submitted form, either a notebook entry or an object.
#  It might be incomplete.
  else {
    $author   = "$main::in{'author'}"; $author =~ s/"/&quot;/g;
    $changes  = "$main::in{'changes'}"; $changes =~ s/"/&quot;/g;
    $kwords   = lc( "$main::in{'keywords'}" ); $kwords =~ s/"/&quot;/g;
    $title    = "$main::in{'title'}"; $title =~ s/"/&quot;/g;
    $caption  = "$main::in{'caption'}";
    $body     = "$main::in{'body'}"; $body =~ s/\r/ /g; $body =~ s/\0//;

    if (( "$mimetype" =~ m;^text/(html|plain)$; ) &&
    ( $main::up{'upfile'}{'size'} > 0 )) {
      $body .= $main::up{'upfile'}{'data'};
    }

    if (( "$mimetype" !~ m;^text/(html|plain)$; ) &&
	( $main::up{'upfile'}{'size'} > 0 )) {
      my $date = time;
      my $strdate = gmtime( $date );
      $date = substr( $date, 0, length( $date ) - 3 ) . '.' .
	      substr( $date, -3 );
      my $filename = "$main::root/$date";
      open( FILE, ">$filename" ) || &InvalidFile();
      binmode( FILE );
      print FILE $main::up{'upfile'}{'data'};
      close( FILE );
      chmod 0666, ( $filename );
      my $objs = 0;
      open( IDX, "<$main::idxfile" ) || &InvalidFile();
      flock( IDX, $main::LOCK_EX ) if ( $main::flock );
      seek( IDX, 0, 0 );
      while ( <IDX> ) {
	next if ( m/^\s*$/ );			# Blank lines
	next if ( m/^\s*#/ );			# Comment lines
      my ( $tag, $name, $type ) = split;
           if ( "$tag" eq "$main::page" ) {
           $base = "$name"; $pagetype = $type;
           }
	$objs++ if ( $tag < 0 );            # Count objects
      }
      $objs++;
      flock( IDX, $main::LOCK_UN ) if ( $main::flock );
      close( IDX );
      open( IDX, ">>$main::idxfile" ) || &InvalidFile();
      flock( IDX, $main::LOCK_EX ) if ( $main::flock );
      print IDX "-$objs $date $mimetype 0\n";
      flock( IDX, $main::LOCK_UN ) if ( $main::flock );
      close( IDX );
      sleep( 1 );
# Check if editor upload of Nob. If so get body and changes.
      if ( $main::in{'editor'} ne '') {
         open( IDX, "<$main::idxfile" ) || &InvalidFile();
         flock( IDX, $main::LOCK_EX ) if ( $main::flock );
         seek( IDX, 0, 0 );
         my @lines = <IDX>;
         foreach ( @lines ) {
         next if ( m/^\s*$/ );			# Blank lines
         next if ( m/^\s*#/ );			# Comment lines
         my ( $tag, $name, $type ) = split;
           if ( "$tag" eq "$main::page" ) {
           $base = "$name"; last;
           }
         }
         flock( IDX, $main::LOCK_UN ) if ( $main::flock );
         close( IDX );

         $filename = "$main::root/$base";

         open( FILE, "<$filename" ) || &InvalidFile();
         while ( <FILE> ) {
         if ( m/^<!-- Keywords: (.*) -->$/ ) { $kwords = $1; }
         elsif ( m/^<!-- Changes: (.*) -->$/ )  { $changes = $1; }
         else { $body .= $_; }
         }
         close( FILE );
      } 
      if ( "$mimetype" =~ m;^image/(jpeg|gif)$; ) {
	$body .= <<EOF;
<p>
<center>
<img src="$main::script?nb=$main::nb&action=view&page=-$objs">
<br><b> $caption </b>
</center>
</p>
EOF
      }
      else {
	$body .= <<EOF;
<p>
Click <a href="$main::script?nb=$main::nb&action=view&page=-$objs">here</a>
to view the object ($caption) you uploaded.
</p>
EOF
      }
      $mimetype = $pagetype;
    }

    if ( $author && $body && $mimetype && $title ) {	# Complete, save it

#  Write the new file out.
      my $strdate = gmtime( time );
      open( FILE, ">$filename" ) || &InvalidFile();
      print FILE <<EOF;
<!-- Author: $author -->
<!-- Date: $strdate (GMT) -->
<!-- Title: $title -->
<!-- Keywords: $kwords -->
<!-- Changes: Modified $strdate (GMT) $author | $changes -->
$body
EOF
      close( FILE );
      chmod 0666, ( $filename );

#  If the MIME type changed, update the index file.
      if ( $update ) {
        open( IDX, "<$main::idxfile" ) || &InvalidFile();
        flock( IDX, $main::LOCK_EX ) if ( $main::flock );
        seek( IDX, 0, 0 );
        my @lines = <IDX>;
        flock( IDX, $main::LOCK_UN ) if ( $main::flock );
        close( IDX );
        open( IDX, ">$main::idxfile" ) || &InvalidFile();
        flock( IDX, $main::LOCK_EX ) if ( $main::flock );
        seek( IDX, 0, 0 );
        foreach ( @lines ) {
          next if ( m/^\s*$/ );			# Blank lines
          next if ( m/^\s*#/ );			# Comment lines
          my ( $tag, $name, $type, $as_is ) = split;
          print IDX ( "$tag" eq "$main::page" ) ?
	    "$tag $base $mimetype $as_is\n" : "$_";
        }
        flock( IDX, $main::LOCK_UN ) if ( $main::flock );
        close( IDX );
      }

#  Redirect the browser to the new entry.
      print <<EOF;
Location: $main::script?nb=$main::nb&action=view&page=$main::page

EOF
      exit $main::RET_NO_ERROR;
    }

# Incomplete entry, retry
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Incomplete Entry</title>
</head>
<body background="$main::backgrnd">
$main::linehorz

<h2>Incomplete Entry</h2>

Every notebook entry must have an author, a title, and some text.<br>
Please complete the missing parts.

EOF
  }

#  Set the default MIME type.
  if ( "$mimetype" ne '' ) {
    foreach ( sort keys %main::mimetypes ) {
      $main::def_type = "$_" if ( "$main::mimetypes{$_}" eq "$mimetype" );
    }
  }

#  The rest of the 'Edit' form.
  if ($notarized == 0 ) {



  print <<EOF;
<form action="$main::script" enctype="multipart/form-data" method=post>
<p>
<pre>
<b>Your name</b>: <input type=text name=author value="$author"
 size=40>
<b>The title</b>: <input type=text name=title value="$title"
 size=40>
<b>Keywords</b> : <input type=text name=keywords value="$kwords"
 size=40> (optional)
</pre>
</p>
<p>
You can <b>type</b> or <b>cut and paste</b> into the text area (it will
scroll to accommodate the input):<br>
<textarea rows=15 cols=75 name=body>$body</textarea><br>
</p>

<p>

Or you can <a href="$main::sketchpad?codebase=$main::codebase&server=$main::server&engine=$main::engine&amp;notebook=$main::nb&page=$main::page">

<b>Draw and add a sketch</b></a> to the end of this page
</p>

<p>
Or you can <b>upload a file</b> (eq. image) to the end of this page: 
<input type=file name=upfile value="" size=20>
<br><input type=text name=caption value="$caption" size=40>
image caption (optional)
</p>
<p>
<b>Choose the MIME type</b> of your input:
<SELECT NAME="mimetype">
EOF

  foreach ( sort keys %main::mimetypes ) {
    print '<OPTION';
    print ' SELECTED' if ( "$_" eq "$main::def_type" );
    print "> $_</OPTION>\n";
  }

  print <<EOF;
</SELECT>
</p>
<p>
Then submit your entry:<br>
<input type=submit name=submit value="Update this entry">
<input type=submit name=reset value="Start over">
<input type=submit name=cancel value="Cancel">
<input type=hidden name=action value="edit">
<input type=hidden name=nb value="$main::nb">
<input type=hidden name=page value="$main::page">
<input type=hidden name=changes value="$changes">
</p>
</form>
</p>
</body>
</html>
EOF
  }
  return;
}

###############################################################################
#
#  ( ) = &ActionNotarize();
#
#  Notarizes an entry in the notebook.
#  Presently Sets flag to 1 in index file to signify this.
#  INCOMPLETE - needs to do MD5 get stamp from notary server.
#
###############################################################################

sub ActionNotarize {

 my ( $author, $body, $changes, $kwords, $mimetype, $title ) =
     ( '', '', '', '', '', '' );
  my ( $base, $filename, $date ) = ( '', '', '');

# Update Index file to indicate page has been notarized

        open( IDX, "<$main::idxfile" ) || &InvalidFile();
        flock( IDX, $main::LOCK_EX ) if ( $main::flock );
        seek( IDX, 0, 0 );
        my @lines = <IDX>;
        flock( IDX, $main::LOCK_UN ) if ( $main::flock );
        close( IDX );
        open( IDX, ">$main::idxfile" ) || &InvalidFile();
        flock( IDX, $main::LOCK_EX ) if ( $main::flock );
        seek( IDX, 0, 0 );
        foreach ( @lines ) {
          next if ( m/^\s*$/ );			# Blank lines
          next if ( m/^\s*#/ );			# Comment lines
          my ( $tag, $name, $type, $as_is ) = split;
          if ( "$tag" eq "$main::page" ) {
	      print IDX "$tag $name $type 1\n";
            $base = $name;
          } else { print IDX "$_" ; }
        }
        flock( IDX, $main::LOCK_UN ) if ( $main::flock );
        close( IDX );


#  Read in file.
    $filename = "$main::root/$base";
    open( FILE, "<$filename" ) || &InvalidFile();
    while ( <FILE> ) {
      if    ( m/^<!-- Author: (.*) -->$/ )   { $author = $1; }
      elsif ( m/^<!-- Date: (.*) -->$/ )     { $date = $1; }
      elsif ( m/^<!-- Title: (.*) -->$/ )    { $title = $1; }
      elsif ( m/^<!-- Keywords: (.*) -->$/ ) { $kwords = $1; }
      elsif ( m/^<!-- Changes: (.*) -->$/ )  { $changes = $1; }
      else { $body .= $_; }
    }
    close( FILE );

#  Modify change history and Write the file back out.
      my $strdate = gmtime( time );
      open( FILE, ">$filename" ) || &InvalidFile();
      print FILE <<EOF;
<!-- Author: $author -->
<!-- Date: $strdate (GMT) -->
<!-- Title: $title -->
<!-- Keywords: $kwords -->
<!-- Changes: Notarized $strdate (GMT) by N/A | $changes -->
$body
<p>
<b>Notarized</b> from start of page to here on $strdate (GMT)<br>
<HR>
EOF
      close( FILE );
      chmod 0666, ( $filename );

#  Instruct the browser to reload this page.
      print <<EOF;
Location: $main::script?nb=$main::nb&action=view&page=$main::page

EOF
      exit $main::RET_NO_ERROR;
}
###############################################################################
#
#  ( ) = &ActionSearch();
#
#  Searches for entries in the notebook.
#
###############################################################################

sub ActionSearch {

#  Extract info if available.
  my ( $casesen, $everything, $hdrtext, $search, $text ) =
     ( '', '', 'Search', '', '' );

  $casesen    = $main::in{'casesen'}	if ( "$main::in{'casesen'}" ne '' );
  $everything = $main::in{'everything'} if ( "$main::in{'everything'}" ne '' );
  $search     = $main::in{'search'}	if ( "$main::in{'search'}" ne '' );
  $search     =~ s/"/&quot;/g;
  if ( "$casesen" ne '1' )    { $casesen = 0; }
  if ( "$everything" ne '1' ) { $everything = 0; }

#  If we have something to search for, open the index file and parse it.
  if ( "$search" ne '' ) {
    $hdrtext .= ' Results';
    
    open( IDX, "<$main::idxfile" ) || &InvalidFile();
    flock( IDX, $main::LOCK_EX ) if ( $main::flock );
    seek( IDX, 0, 0 );

    while ( <IDX> ) {
      next if ( m/^\s*$/ );		# Blank lines
      next if ( m/^\s*#/ );		# Comment lines

      my ( $tag, $name, $type, $as_is ) = split;

      next if ( $tag < 0 );		# Skip notebook objects
      next if ( "$name" eq '' );	# Skip deleted pages

      $name = "$main::root/$name" if ( $name !~ m/^\// );
      open( FILE, "<$name" ) || &InvalidFile();

#  Scan the file for the author name, creation date, and title.
      my ( $author, $body, $date, $kwords, $title ) = ( '', '', '', '' );
      while ( <FILE> ) {
	if    ( m/^<!-- Author: (.*) -->$/ )   { $author = $1; }
	elsif ( m/^<!-- Date: (.*) -->$/ )     { $date = $1; }
	elsif ( m/^<!-- Keywords: (.*) -->$/ ) { $kwords = $1; }
	elsif ( m/^<!-- Title: (.*) -->$/ )    { $title = $1; }
	else				       { $body .= $_; }
      }
      close( FILE );

#  Search this entry.
      my ( $found ) = ( 0 );
      $found = ( $kwords =~ m/$search/io );
      if (( ! $found ) && ( "$everything" eq '1' )) {
	if ( "$casesen" eq '1' ) {
	  $found = ( $body =~ m/$search/o );
	}
	else {
	  $found = ( $body =~ m/$search/io );
	}
      }

      if ( $found ) {
	$text .= "<li><a href=\"$main::script?nb=$main::nb&action=view&" .
		 "page=$tag\">$title\n($author)</a>\n";
      }
    }

    flock( IDX, $main::LOCK_UN ) if ( $main::flock );
    close( IDX );
  }

#  Build the page.
  print <<EOF;
Content-type: text/html

<html>
<head>
<title>$hdrtext</title>
</head>
<body background="$main::backgrnd">
<center><h2>$main::title</h2></center>
EOF

#  The search header.
  print <<EOF;
<form action="$main::script" method=post>
<p>
<center>
<b>Search for</b>: <input type=text name=search value="$search" size=35>
<input type=submit name=submit value="Search">
<input type=submit name=cancel value="Cancel">
<input type=hidden name=action value="search">
<input type=hidden name=nb value="$main::nb">
<input type=hidden name=page value="$main::page">
<br>
EOF
  print '<input type=checkbox name=everything value=1';
  if ( "$everything" eq '1' ) { print ' checked'; }
  print '> Search in entries text also';
  print '<input type=checkbox name=casesen value=1';
  if ( "$casesen" eq '1' ) { print ' checked'; }
  print '> Case sensitive';
  print <<EOF;
</center>
<center>
Keywords are always searched (case-insensitive).
</center>
</p>
</form>
</body>
</html>
EOF

#  The results of the search, if we found anything.
  print "$main::linehorz\n<ul>$text</ul></body>\n</html>\n";

  return;
}

###############################################################################
#
#  ( ) = &ActionToc();
#
#  Builds a table of contents for the notebook.
#
###############################################################################

sub ActionToc {

#  Open the index file and parse it.
  open( IDX, "<$main::idxfile" ) || &InvalidFile();
  flock( IDX, $main::LOCK_EX ) if ( $main::flock );
  seek( IDX, 0, 0 );

#  Open the index file and parse it.
  my $text = '';
  my $text2= '';
  while ( <IDX> ) {
    next if ( m/^\s*$/ );		# Blank lines
    next if ( m/^\s*#/ );		# Comment lines

    my ( $tag, $name, $type, $as_is ) = split;

#   next if ( $tag < 0 );		# Skip notebook objects
    if ( $tag < 0 ) {          # NEW -- list objects at the end
      $text2 .= "<li><a href=\"$main::script?nb=$main::nb&action=view&" .
	       "page=$tag\">$main::script?nb=$main::nb&action=view&page=$tag</a> $type\n";
    }

    if ( "$name" eq '' ) {		# Deleted page
      $text .= "<li>Deleted page\n";
    }
    elsif ( $tag > 0) {
      $name = "$main::root/$name" if ( $name !~ m/^\// );
      open( FILE, "<$name" ) || &InvalidFile();

#  Scan the file for the author name, creation date, and title.
      my ( $author, $title, $date ) = ( '', '', '' );
      while ( <FILE> ) {
	if    ( m/^<!-- Author: (.*) -->$/ ) { $author = $1; }
	elsif ( m/^<!-- Date: (.*) -->$/ )   { $date = $1; }
	elsif ( m/^<!-- Keywords: .* -->$/ ) { ; }
	elsif ( m/^<!-- Title: (.*) -->$/ )  { $title = $1; }
      }
      close( FILE );
      $text .= "<li><a href=\"$main::script?nb=$main::nb&action=view&" .
	       "page=$tag\">$title\n($author)\n</a>[Last Modified $date]\n";
    }
  }

  flock( IDX, $main::LOCK_UN ) if ( $main::flock );
  close( IDX );

#  Build the page.
  print <<EOF;
Content-type: text/html

<html>
<head>
<title>Table of Contents</title>
</head>
<body background="$main::backgrnd">
<center><h2>$main::title</h2></center>
$main::linehorz
<h4>Table of Contents</h4>
<ol>
$text
</ol>
<hr>
<center><b> Other uploaded notebook objects</b></center>
<ul>
$text2
</ul>
</body>
</html>
EOF

  return;
}

###############################################################################
#
#  ( ) = &ActionView();
#
#  Views an entry from the notebook.
#
###############################################################################

sub ActionView {

  my ( $filename, $filetype, $objs, $pageno, $pages ) = ( '', '', 0, 0, 0 );
  my  $flag = 0 ;
#  Open the index file and parse it.
  open( IDX, "<$main::idxfile" ) || &InvalidFile();
  flock( IDX, $main::LOCK_EX ) if ( $main::flock );
  seek( IDX, 0, 0 );
  while ( <IDX> ) {

    next if ( m/^\s*$/ );			# Blank lines
    next if ( m/^\s*#/ );			# Comment lines

    my ( $tag, $name, $type, $as_is ) = split;
    
    if ( $tag < 0 ) {				# Object
      $objs++;
      if ( "$tag" eq "$main::page" ) {
	$pageno   = $objs;
	$filename = $name;
	$filetype = lc( $type );
      }
    }
    elsif ( $tag > 0 ) {			# Page
      $pages++;
      if (( "$main::page" eq 'first' ) ||	# Looking for first file
	  ( "$tag" eq "$main::page" )) {	# Found a matching tag
	$main::page = '';
	$pageno     = $pages;
	$filename   = $name;
	$filetype   = lc( $type );
      $flag = $as_is ;
      }
      elsif ( "$main::page" eq 'last' ) {	# Looking for last file
	$pageno   = $pages;
	$filename = $name;
	$filetype = lc( $type );
      $flag = $as_is ;
      }
    }
  }
  flock( IDX, $main::LOCK_UN ) if ( $main::flock );
  close( IDX );
  
 if ( $flag == 1 ) {
      $main::noedit = 1;
      $main::nodelete = 1;
 }

#  Check that we found our page.
  my ( $author, $changes, $date, $kwords, $text, $title ) =
     ( '', '', '', '', '', '' );
  if ( $pageno == 0 ) {
    if (( $pages == 0 ) &&
	(( $main::page == 0 ) || ( "$main::page" eq 'last' ))) {
      ; 					# Empty notebook, OK
    }
    else {					# Request out of bounds
      print <<EOF;
Content-type: text/html

<html>
<head>
<title>Request Error</title>
</head>
<body background="$main::backgrnd">
<h1>Request Error</h1>
<p><b>The requested notebook page or object does not exist.</b></p>
Please use your browser to go back.<hr>
</body>
</html>
EOF
      exit $main::RET_BAD_PAGE;
    }
  }

#  Check that the object or page has not been deleted.
  my $deleted = 0;
  if ( "$filename" eq '' ) {
    if ( $main::page < 0 ) {			# Requested a deleted object
      print "Content-type: text/html\n\n";
      exit $main::RET_NO_ERROR;
    }
    elsif ( $pages > 0 ) {			# Requested a deleted page
      $text  = '<p><center><b>This page was deleted.</b></center></p>';
      $title = 'Deleted page';
      $deleted = 1;
    }
  }

  else {
    $filename = "$main::root/$filename" if ( $filename !~ m/^\// );
    open( FILE, "<$filename" ) || &InvalidFile();

#  For HTML and plain text files, extract the author name, creation date,
#  title, and change history.
    if ( "$filetype" =~ m;^text/(html|plain)$; ) {
      while ( <FILE> ) {
	if    ( m/^<!-- Author: (.*) -->$/ )   { $author = $1; }
	elsif ( m/^<!-- Date: (.*) -->$/ )     { $date = $1; }
	elsif ( m/^<!-- Title: (.*) -->$/ )    { $title = $1; }
	elsif ( m/^<!-- Keywords: (.*) -->$/ ) { $kwords = $1; }
	elsif ( m/^<!-- Changes: (.*) -->$/ )  { $changes = $1; }
	else { $text .= $_; }
      }
      close( FILE );
    }

#  All other files, send as is and exit.
    else {
      print "Content-type: $filetype\n\n";
      $| = 1;
      binmode( FILE );
      binmode( STDOUT );
      while ( <FILE> ) { print; }
      close( FILE );
      exit $main::RET_NO_ERROR;
    }
  }

#  Have a valid page file, build the page.
#  NEW added JavaScript to fire up external annotate window

  my $wintitle = ( "$title" eq '' ) ? "Page $pageno of $pages" :
		 "$title ($pageno/$pages)";
  print <<EOF;
Content-type: text/html

<html>
<head>
<title>$wintitle</title>
<html>
<head>
<title>Notebook page $pageno</title>
</HEAD>
<body background="$main::backgrnd"> 
<center><h3>$main::title - page $pageno of $pages</h3></center>
EOF

  if ( $main::nopanel == 0 ) {
#  NEW Refresh Control Panel
#  Print script that raises the button(s) on external control panel
print <<EOF;
<script>
Ctrl = window.open("", "Ctrl", "width=180,height=160,scrollbars=0,resizable=0");
Ctrl.opener.name = "creator";
Ctrl.document.writeln("<HTML><HEAD><TITLE>$main::title</TITLE></HEAD>");
Ctrl.document.writeln("<BODY BGCOLOR=#C0C0C0>");
Ctrl.document.writeln("<a "); 
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=toc&page=$pageno\\" TARGET='creator' onMouseDown=\\"content.src='$main::newgifs/Contents1.gif'; return true;\\" ><IMG "); 
Ctrl.document.writeln("SRC=\\"$main::newgifs/Contents.gif\\" Border=0 name=content></a><nobr><a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=search&page=$pageno\\" TARGET='creator' onMouseDown=\\"find.src='$main::newgifs/Search1.gif'; return true;\\"><IMG "); 
Ctrl.document.writeln("SRC=\\"$main::newgifs/Search.gif\\" Border=0 name=find></a><br> ");
EOF
#  If not first page, put a 'Prev' link.
  if ( $pageno > 1 ) {
    my $target = $pageno - 1;
print <<EOF;
Ctrl.document.writeln("<a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=view&page=$target\\" TARGET='creator' onMouseDown=\\"previous.src='$main::newgifs/Previous1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/Previous.gif\\" Border=0 name=previous></a");
EOF
  }
  else {
    print <<EOF;
Ctrl.document.writeln("<IMG SRC=\\"$main::newgifs/Previous2.gif\\" Border=0 name=previous");
EOF
  }
  if ( $pageno < $pages ) {
    my $target = $pageno + 1;
    print <<EOF;
Ctrl.document.write("><a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=view&page=$target\\" TARGET='creator' onMouseDown=\\"next.src='$main::newgifs/Next1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/Next.gif\\" Border=0 name=next></a><br> ");
EOF
  }
  else {
    print <<EOF;
Ctrl.document.writeln("><IMG SRC=\\"$main::newgifs/Next2.gif\\" Border=0 name=next><br> ");
EOF
  }
print <<EOF;
Ctrl.document.writeln("<a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=view&page=1\\" TARGET='creator' onMouseDown=\\"first.src='$main::newgifs/First1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/First.gif\\" Border=0 name=first></a><a "); 
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=view&page=last\\" TARGET='creator' onMouseDown=\\"last.src='$main::newgifs/Last1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/Last.gif\\" Border=0 name=last></a><br> ");
EOF
# allow add function to be disabled in config
  if ( $main::noadd == 0 ) {
    print <<EOF;
Ctrl.document.writeln("<a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=add&page=$pageno\\" TARGET='creator' onMouseDown=\\"add.src='$main::newgifs/Add1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/Add.gif\\" Border=0 name=add></a");
EOF
  }
  else {
    print <<EOF;
Ctrl.document.writeln("<IMG SRC=\\"$main::newgifs/Add2.gif\\" Border=0 name=add");
EOF
  }
# allow delete function to be disabled in config
  if ( $main::nodelete == 0 ) {
    print <<EOF;
Ctrl.document.write("><a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=delete&page=$pageno\\" TARGET='creator' onMouseDown=\\"del.src='$main::newgifs/Delete1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/Delete.gif\\" Border=0 name=del></a><br> ");
EOF
  }
  else {
    print <<EOF;
Ctrl.document.writeln("><IMG SRC=\\"$main::newgifs/Delete2.gif\\" Border=0 name=del><br> ");
EOF
  }
# allow edit function to be disabled in config
  if ( $main::noedit == 0 ) {
    print <<EOF;
Ctrl.document.writeln("<a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=edit&page=$pageno\\" TARGET='creator' onMouseDown=\\"edit.src='$main::newgifs/Edit1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/Edit.gif\\" Border=0 name=edit");
EOF
  }
  else {
    print <<EOF;
Ctrl.document.writeln("<IMG SRC=\\"$main::newgifs/Edit2.gif\\" Border=0 name=edit");
EOF
  }
# allow annotate function to be disabled in config
  if ( $main::noannotate == 0 ) {
    print <<EOF;
Ctrl.document.write("><a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=annotate&page=$pageno\\" TARGET='creator' onMouseDown=\\"annotate.src='$main::newgifs/Annotate1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/Annotate.gif\\" Border=0 name=annotate></a><br> ");
EOF
  }
  else {
    print <<EOF;
Ctrl.document.writeln("><IMG SRC=\\"$main::newgifs/Annotate2.gif\\" Border=0 name=annotate><br> ");
EOF
  }
# allow notarize function to be disabled in config
  if ( $main::nonotarize == 0 ) {
    print <<EOF;
Ctrl.document.write("<a ");
Ctrl.document.writeln("href=\\"$main::script?nb=$main::nb&action=notarize&page=$pageno\\" TARGET='creator' onMouseDown=\\"notarize.src='$main::newgifs/Notarize1.gif'; return true;\\"><IMG ");
Ctrl.document.writeln("SRC=\\"$main::newgifs/Notarize.gif\\" Border=0 name=notarize ");
EOF
  }
  else {
    print <<EOF;
Ctrl.document.writeln("<IMG SRC=\\"$main::newgifs/Notarize2.gif\\" Border=0 name=notarize");
EOF
  }
# Finally put in the close button
print <<EOF;
Ctrl.document.write("><a ");
Ctrl.document.writeln("href=\\"../enote/stop.html\\" onMouseDown=\\"exit.src='$main::newgifs/Close1.gif'; self.close(); return true;\\"><IMG SRC=\\"$main::newgifs/Close.gif\\" Border=0 name=exit></a> ");
Ctrl.document.writeln("</BODY></HTML>");
Ctrl.document.close();
</script>
EOF
# END refreshCtrl JavaScript block
  }

#  Print the navigation bar at the top.
#  Experiment with living without Navigation bar

   if ( $main::nobar != 1 ) {

#  Print the left end, top line.
  print <<EOF;
<center>
<img
 src="$main::gifs/lend.gif" height=$main::btn_ht width=$main::lend_wd
 border=0 alt=""
EOF
  print " >";

#  Put the 'Add' link.
  if ( $main::noadd == 0 ) {
  print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=add&page=$pageno"
 onMouseOver="add.src='$main::gifs/add2.gif';
 window.status='Add a page'; return true;"
 onMouseOut="add.src='$main::gifs/add1.gif'; return true;"><img
 src="$main::gifs/add1.gif" height=$main::btn_ht width=$main::add_wd
 border=0 alt="Add" name=add
EOF
  print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/add0.gif" height=$main::btn_ht width=$main::add_wd 
 border=0 alt="Add"
EOF
    print " >";
  }

#  Put the 'Edit' link.
  if (( $pageno > 0 ) && ( ! $deleted ) && ( $main::noedit == 0 )) {
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=edit&page=$pageno"
 onMouseOver="edit.src='$main::gifs/edit2.gif';
 window.status='Edit current page'; return true;"
 onMouseOut="edit.src='$main::gifs/edit1.gif'; return true;"><img
 src="$main::gifs/edit1.gif" height=$main::btn_ht width=$main::edit_wd
 border=0 alt="Edit" name=edit
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/edit0.gif" height=$main::btn_ht width=$main::edit_wd 
 border=0 alt="Edit"
EOF
    print " >";
  }

#  Put the 'Delete' link.
  if (( $pageno > 0 ) && ( ! $deleted ) && ( $main::nodelete == 0 )) {
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=delete&page=$pageno"
 onClick="return confirm('     Warning: Irreversible operation\\n\\nPress OK to Delete current document \\n\\nPress Cancel otherwise\\n\\n');"
 onMouseOver="del.src='$main::gifs/del2.gif';
 window.status='Delete current page'; return true;"
 onMouseOut="del.src='$main::gifs/del1.gif'; return true;"><img
 src="$main::gifs/del1.gif" height=$main::btn_ht width=$main::del_wd
 border=0 alt="Delete" name=del
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/del0.gif" height=$main::btn_ht width=$main::del_wd
 border=0 alt="Delete"
EOF
    print " >";
  }

#  Put the 'Annotate' link.
  if (( $pageno > 0 ) && ( ! $deleted ) && ( $main::noannotate == 0 )) {
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=annotate&page=$pageno"
 onMouseOver="anno.src='$main::gifs/anno2.gif';
 window.status='Annotate current page'; return true;"
 onMouseOut="anno.src='$main::gifs/anno1.gif'; return true;"><img
 src="$main::gifs/anno1.gif" height=$main::btn_ht width=$main::anno_wd  
 border=0 alt="Annotate" name=anno
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/anno0.gif" height=$main::btn_ht width=$main::anno_wd  
 border=0 alt="Annotate" 
EOF
    print " >";
  }

#  Put the 'Notarize' link.
  if (( $pageno > 0 ) && ( ! $deleted ) && ( $main::nonotarize == 0 )) {
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=notarize&page=$pageno"
 onClick="return confirm('     Warning: Irreversible operation\\n\\nPress OK to Notarize current document \\n\\nPress Cancel otherwise\\n\\n');"
 onMouseOver="notar.src='$main::gifs/notar2.gif';
 window.status='Notarize current page'; return true;"
 onMouseOut="notar.src='$main::gifs/notar1.gif'; return true;"><img
 src="$main::gifs/notar1.gif" height=$main::btn_ht width=$main::notar_wd
 border=0 alt="Notarize" name=notar 
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/notar0.gif" height=$main::btn_ht width=$main::notar_wd
 border=0 alt="Notarize"
EOF
    print " >";
  }

#  Print the right end, top line.
  print <<EOF;
<img
 src="$main::gifs/rend.gif" height=$main::btn_ht width=$main::rend_wd
 border=0 alt=""><br>
EOF

#  Print the left end, bottom line.
  print <<EOF;
<img
 src="$main::gifs/lend.gif" height=$main::btn_ht width=$main::lend_wd
 border=0 alt=""
EOF
  print " >";

#  If not first page, put a 'First' link.
  if ( $pageno > 1 ) {
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=view&page=1"
 onMouseOver="first.src='$main::gifs/first2.gif';
 window.status='Go to first page'; return true;"
 onMouseOut="first.src='$main::gifs/first1.gif'; return true;"><img
 src="$main::gifs/first1.gif" height=$main::btn_ht width=$main::first_wd  
 border=0 alt="First Page" name=first
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/first0.gif" height=$main::btn_ht width=$main::first_wd  
 border=0 alt="First Page" 
EOF
    print " >";
  }

#  If not first page, put a 'Prev' link.
  if ( $pageno > 1 ) {
    my $target = $pageno - 1;
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=view&page=$target"
 onMouseOver="prev.src='$main::gifs/prev2.gif';
 window.status='Go to previous page'; return true;"
 onMouseOut="prev.src='$main::gifs/prev1.gif'; return true;"><img
 src="$main::gifs/prev1.gif" height=$main::btn_ht width=$main::prev_wd  
 border=0 alt="Previous Page" name=prev
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/prev0.gif" height=$main::btn_ht width=$main::prev_wd  
 border=0 alt="Previous Page" 
EOF
    print " >";
  }

#  If not first page, put a 'Next' link.
  if ( $pageno < $pages ) {
    my $target = $pageno + 1;
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=view&page=$target"
 onMouseOver="next.src='$main::gifs/next2.gif';
 window.status='Go to next page'; return true;"
 onMouseOut="next.src='$main::gifs/next1.gif'; return true;"><img
 src="$main::gifs/next1.gif" height=$main::btn_ht width=$main::next_wd  
 border=0 alt="Next Page" name=next
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/next0.gif" height=$main::btn_ht width=$main::next_wd  
 border=0 alt="Next Page" 
EOF
    print " >";
  }

#  If not first page, put a 'Last' link.
  if ( $pageno < $pages ) {
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=view&page=last"
 onMouseOver="last.src='$main::gifs/last2.gif';
 window.status='Go to last page'; return true;"
 onMouseOut="last.src='$main::gifs/last1.gif'; return true;"><img
 src="$main::gifs/last1.gif" height=$main::btn_ht width=$main::last_wd  
 border=0 alt="Last Page" name=last
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/last0.gif" height=$main::btn_ht width=$main::last_wd  
 border=0 alt="Last Page"
EOF
    print " >";
  }

#  Put the 'Table of Contents' link.
  if ( $pages > 0 ) {
    print <<EOF;
<a
 href="$main::script?nb=$main::nb&action=toc&page=$pageno"
 onMouseOver="toc.src='$main::gifs/toc2.gif';
 window.status='Table of Contents'; return true;"
 onMouseOut="toc.src='$main::gifs/toc1.gif'; return true;"><img
 src="$main::gifs/toc1.gif" height=$main::btn_ht width=$main::toc_wd  
 border=0 alt="Table of Contents" name=toc
EOF
    print " ></a>";
  }
  else {
    print <<EOF;
<img
 src="$main::gifs/toc0.gif" height=$main::btn_ht width=$main::toc_wd  
 border=0 alt="Table of Contents" 
EOF
    print " >";
  }

#  Put the 'Search' link.
  if ( $pages > 0 ) {
    print <<EOF;    
<a      
 href="$main::script?nb=$main::nb&action=search&page=$pageno"
 onMouseOver="srch.src='$main::gifs/srch2.gif';
 window.status='Search'; return true;" 
 onMouseOut="srch.src='$main::gifs/srch1.gif'; return true;"><img
 src="$main::gifs/srch1.gif" height=$main::btn_ht width=$main::srch_wd
 border=0 alt="Search" name=srch
EOF
    print " ></a>";
  }
  else { 
    print <<EOF;
<img
 src="$main::gifs/srch0.gif" height=$main::btn_ht width=$main::srch_wd
 border=0 alt="Search"
EOF
    print " >";
  } 

#  Print the right end, bottom line.
  print <<EOF;
<img
 src="$main::gifs/rend.gif" height=$main::btn_ht width=$main::rend_wd
 border=0 alt="">
</center>
<HR>
EOF
  }
# End nobar block

# Use drop down menu rather than single line for change history
  $changes =~ s/\|/<OPTION>/g;
print <<EOF; 
<TABLE FRAME=ALL BORDER=0 RULES=Basic CELLPADDING=0 HEIGHT=20>
<TR ALIGN="left" VALIGN="top">
<TD><b>Date and Author(s)</b>
<TD><FORM><SELECT SIZE=1 NAME="History">
<OPTION>$changes
</SELECT> 
</FORM>
</TABLE>
EOF
  print "$main::lineview <br>\n";  
  print "\n<center><h3>$title</h3></center>" if ( "$title" ne '' );
  print "\n<p>";
  print "\n<pre>"  if ( "$filetype" eq 'text/plain' );
  #
  # Wrap plain text within 80 characters
  #if ( "$filetype" eq 'text/plain' ){
  #  print "\n"; print wrap("", "", $text);
  #}
  #else {
    print "\n$text";
  #}
  print "\n</pre>" if ( "$filetype" eq 'text/plain' );
  print "\n</p>";

#  Put the trailer.
  print "\n</body>\n</html>\n";

  return;
}

###############################################################################
#
#  Auxiliary support routines
#
###############################################################################

###############################################################################
#
#  ( ) = &InvalidFile();
#
#  Prints an error message for invalid file and exits the script.
#
###############################################################################

sub InvalidFile {

  my $filename = @_ ? " on @_[0]" : '';

  print <<EOF;
Content-type: text/html

<html>
<head>
<title>File Error</title>
</head>
<body background="$main::backgrnd">
<h1>File Error</h1>
<p><b>Configuration error: the server encountered a file error or an
invalid file request$filename.</b></p>
Please use your browser to go back.<hr>
</body>
</html>
EOF
  exit $main::RET_BAD_FILE;

}

###############################################################################
#
#  $num = &ParseCGIParms();
#
#  Parse the CGI parameters.
#
###############################################################################

sub ParseCGIParms {

  my $ret = 0;
  my $read_offset = 0;
  my $read_length = $ENV{'CONTENT_LENGTH'};

  $main::in = ''; @main::in = (); %main::in = ();
  $main::up = ''; @main::up = (); %main::up = ();
  if ( $ENV{'REQUEST_METHOD'} eq 'GET'  ) {
    $main::in = $ENV{'QUERY_STRING'}; }
  elsif ( $ENV{'REQUEST_METHOD'} eq 'POST' ) {
    binmode( STDIN );

    while($ret = read( STDIN, $main::in, $read_length-$read_offset, $read_offset)) {
      $read_offset += $ret;
    }
  }
  if ( $ENV{'CONTENT_TYPE'} =~ m|^multipart/form-data| ) {
    my ( $s7 ) = $ENV{'CONTENT_TYPE'} =~ /boundary=(\S+)/;
    $s7 = "--$s7"; my $i2 = length( $s7 ); return 0 if $i2 <= 0;
    $_ = index( $main::in, 'Content-' ) - $i2;
    my $s4 = ( $_ > 0 ) ? substr( $main::in, $i2, $_ ) : '';
    my $i1 = length( $s4 ); return 0 if $i1 <= 0;
    my $s5 = '^Content-[Dd]isposition: form-data; name="(.+?)"';
    $s5 .= '(|; filename="(.*?)")$';
    my $s6 = '^Content-[Tt]ype: (\S+)$';
    my ( $body ) = ( '' );
    foreach ( split( /$s7/, $main::in )) {
      next if ( ! m/^$s4/o );
      my $s3 = substr( $_, 0, index( $_, "$s4$s4" ) + 2 * $i1 );
      next if ( "$s3" eq '' );
      my ( $key, $b1, $mime, $name ) = ( '', 0, '', '' );
      my ( $s1, $s2 ) = ( 0, index( $s3, $s4 ));
      while ( $s2 >= 0 ) {
	my $s = substr( $s3, $s1, $s2 - $s1 );
	$s1 = $s2 + $i1; $s2 = index( $s3, $s4, $s1 );
	if ( $s =~ m-$s5-o ) {
	  $key = "$1"; $b1 = ( "$2" ne '' ); $name = "$3"; }
	elsif ( $s =~ m-$s6-o ) { $mime = "$1"; }
	else { ; }}
      next if ( "$key" eq '' );
      if ( ! $b1 ) {
	$main::in{$key} .= "\0" if ( defined( $main::in{$key} ));
	$main::in{$key} .= substr( $_, $s1, -$i1 ); next; }
      if ( "$name" ne '' ) {
	$main::up{$key}{'mime'} = "$mime"; $main::up{$key}{'name'} = "$name";
	$main::up{$key}{'data'} = substr( $_, $s1, -$i1 );
	$main::up{$key}{'size'} = length( $main::up{$key}{'data'} ); next; }}}
  else {
    @main::in = split( /[&;]/, $main::in ); my $i;
    foreach $i (0 .. $#main::in) {
      $main::in[$i] =~ s/\+/ /g;
      my ($key, $val) = split( /=/, $main::in[$i], 2 );
      $key =~ s/%(..)/pack('c',hex($1))/ge;
      $val =~ s/%(..)/pack('c',hex($1))/ge;
      $main::in{$key} .= "\0" if (defined($main::in{$key}));
      $main::in{$key} .= $val; }}
  return scalar(@main::in); 
}

1;

__END__

